%using Babel;
%using dragonangel.phpProject.Language.DSL.Parser;
%using Babel.Parser;

%namespace Babel.Lexer


%x COMMENT
%x STRINGSIMPLEQUOTE
%x STRINGQUOTE
%x TYPEDEFINITION
%x TYPEINHERITANCE
%x MEMBERREFERENCE


%{
		public int GetIdToken(string txt)
		{
			return (int)dragonangel.phpProject.Language.TokenMapping.GetToken(txt);            
		}
       
       internal void LoadYylval()
       {
           yylval.str = tokTxt;
           yylloc = new LexLocation(tokLin, tokCol, tokLin, tokECol);
       }
       
       public override void yyerror(string s, params object[] a)
       {
           if (handler != null) handler.AddError(s, tokLin, tokCol, tokLin, tokECol);
       }
%}


White0          [ \t\r\f\v]
White           {White0}|\n
optionalWhite   {White}*

CmntStart    \/\*
CmntEnd      \*\/
ABStar       [^\*\n]*
StringSimpleQuote \'
StringQuote	 \"
identifier	[a-zA-Z_][a-zA-Z0-9_]*

%%

class											{ BEGIN(TYPEDEFINITION); return GetIdToken(yytext); }
interface										{ BEGIN(TYPEDEFINITION); return GetIdToken(yytext); }
<TYPEDEFINITION>{identifier}					{	if(yytext=="extends")
													{
														BEGIN(TYPEINHERITANCE);
														return (int)Tokens.INHERIT;
													}
													if(yytext=="implements")
													{
														BEGIN(TYPEINHERITANCE);
														return (int)Tokens.IMPLEMENT;
													}
													return (int)Tokens.TYPEIDENTIFIER;
												}
<TYPEDEFINITION>{optionalWhite}					{ break; }
<TYPEDEFINITION>\{								{ BEGIN(INITIAL); return '{'; }
<TYPEINHERITANCE>{identifier}					{ 
													if(yytext=="implements")
														return (int)Tokens.IMPLEMENT;
													return (int) Tokens.TYPEREFERENCEIDENTIFIER;
												}
<TYPEINHERITANCE>,								{ return ','; }
<TYPEINHERITANCE>{optionalWhite}				{ break; }
<TYPEINHERITANCE>\{								{ BEGIN(INITIAL); return '{'; }

{StringSimpleQuote}([^\\\']*(\\\')*(\\[^'])*)*{StringSimpleQuote}			{ return (int)Tokens.SIMPLEQUOTESTRING; }
{StringSimpleQuote}													{ BEGIN(STRINGSIMPLEQUOTE); return (int)Tokens.SIMPLEQUOTE; }
<STRINGSIMPLEQUOTE>([^\\\']*(\\\')*\\[^']*)*								{ return (int)Tokens.SIMPLEQUOTESTRING; }
<STRINGSIMPLEQUOTE>{StringSimpleQuote}								{ BEGIN(INITIAL); return (int)Tokens.SIMPLEQUOTE; }

{StringQuote}([^\\\"]*(\\\")*(\\[^"])*)*{StringQuote}						{ return (int)Tokens.QUOTESTRING; }
{StringQuote}														{ BEGIN(STRINGQUOTE); return (int)Tokens.QUOTE; }
<STRINGQUOTE>([^\\\"]*(\\\")*\\[^"]*)*										{ return (int)Tokens.QUOTESTRING; }
<STRINGQUOTE>([^\\\"]*(\\\")*\\[^"]*)*{StringQuote}						{ BEGIN(INITIAL); return (int)Tokens.QUOTE; }

<MEMBERREFERENCE>{identifier}	{ BEGIN(INITIAL);return (int)Tokens.MEMBERIDENTIFIER; }

\<\?php					  { return (int)Tokens.ALTERNATE_BEGIN_CODE; }
\<\?					  { return (int)Tokens.BEGIN_CODE; }
\?\>					  { return (int)Tokens.END_CODE; }
\${identifier}			  { return GetIdToken(yytext); }
{identifier}			  { return GetIdToken(yytext); }
[0-9]+                    { return (int)Tokens.NUMBER; }
\/\/[^\r\n]*			  { return (int)Tokens.LEX_COMMENT; }
;                         { return (int)';';    }
-\>						  { BEGIN(MEMBERREFERENCE); return (int)Tokens.ACCESSOR; }
::						  { BEGIN(MEMBERREFERENCE); return (int)Tokens.CLASSACCESSOR; }
:						  { return (int)':';	}
,                         { return (int)',';    }
\(                        { return (int)'(';    }
\)                        { return (int)')';    }
\{                        { return (int)'{';    }
\}                        { return (int)'}';    }
\[						  { return (int)'[';	}
\]						  { return (int)']';	}
=\>						  { return (int)Tokens.MAP; }
=                         { return (int)'=';    }
\^                        { return (int)'^';    }
\+                        { return (int)'+';    }
\-                        { return (int)'-';    }
\*                        { return (int)'*';    }
\/                        { return (int)'/';    }
\!                        { return (int)'!';    }
===                        { return (int)Tokens.TYPEEQ;  }
==                        { return (int)Tokens.EQ;  }
\!=                       { return (int)Tokens.NEQ;   }
\>                        { return (int)Tokens.GT; }
\>=                       { return (int)Tokens.GTE;    }
\<                        { return (int)Tokens.LT;     }
\<=                       { return (int)Tokens.LTE;    }
\.=						  { return (int)Tokens.SELFCONCAT; }
\&\&                      { return (int)Tokens.AMPAMP; }
\&                        { return (int)'&';    }
\|                        { return (int)'|';    }
\|\|                      { return (int)Tokens.BARBAR; }
\.                        { return (int)'.';    }

{CmntStart}{ABStar}\**{CmntEnd} { return (int)Tokens.LEX_COMMENT; } 
{CmntStart}{ABStar}\**          { BEGIN(COMMENT); return (int)Tokens.LEX_COMMENT; }
<COMMENT>\n                     |                                
<COMMENT>{ABStar}\**            { return (int)Tokens.LEX_COMMENT; }                                
<COMMENT>{ABStar}\**{CmntEnd}   { BEGIN(INITIAL); return (int)Tokens.LEX_COMMENT; }


{White0}+                  { return (int)Tokens.LEX_WHITE; }
(\r*\n\r*)+                { return (int)Tokens.LEX_WHITE; }
.                          { yyerror("illegal char"); return (int)Tokens.LEX_ERROR; }

%{
                      LoadYylval();
%}

%%

/* .... */
