%using Microsoft.VisualStudio.TextManager.Interop
%using dragonangel.phpProject.Language.DSL
%using Babel.Parser
%namespace dragonangel.phpProject.Language.DSL.Parser
%valuetype LexValue
%partial

/* %expect 5 */


%union {
    public string str;
}



%{
%}

%token IDENTIFIER NUMBER VARIABLE TYPEIDENTIFIER TYPEREFERENCEIDENTIFIER MEMBERIDENTIFIER
%token BEGIN_CODE END_CODE ALTERNATE_BEGIN_CODE
%token KWCLASS KWFUNCTION KWINTERFACE KWNULL KWABSTRACT
%token KWIF KWELSE KWELSEIF KWWHILE KWFOR KWCONTINUE KWBREAK KWRETURN KWFOREACH KWAS KWECHO KWSWITCH KWCASE KWDEFAULT
%token KWSTATIC KWINT KWVOID KWGLOBAL
%token KWPUBLIC KWPROTECTED KWPRIVATE KWVAR
%token KWINCLUDE KWINCLUDEONCE KWREQUIRE KWREQUIREONCE
%token KWTHIS KWPARENT KWNEW KWTHROW
%token ACCESSOR CLASSACCESSOR INHERIT IMPLEMENT
%token SIMPLEQUOTE QUOTE
%token SIMPLEQUOTESTRING QUOTESTRING

%token KWTRUE KWFALSE
%token KWARRAY BUILTIN_FUNCTION


  // %token ',' ';' '(' ')' '{' '}' '=' 
  // %token '+' '-' '*' '/' '!' '&' '|' '^'

%token EQ NEQ TYPEEQ GT GTE LT LTE AMPAMP BARBAR KWINSTANCEOF SELFCONCAT MAP
%token maxParseToken 
%token LEX_WHITE LEX_COMMENT LEX_ERROR

%left ';' '.' '('
%left '*' '/' '%' AMPAMP
%left '+' '-' BARBAR
%left '['
%left ACCESSOR CLASSACCESSOR
%left '='
%left EQ NEQ TYPEEQ GT GTE LT LTE
%%

Program
    : Declarations				{ PushCodeFile(); }
    ;

Declarations
    : BEGIN_CODE Declaration END_CODE
	| ALTERNATE_BEGIN_CODE Declaration END_CODE
    | Declaration error         { CallHdlr("Expected Declaration", @2); }
    | 
    ;
    
Declaration   /* might need an init action for symtab init here */
	: Declaration_
	| Declaration Declaration_
	;

Declaration_
	: ExclusiveDeclaration
    | Statements
    ;
    
ExclusiveDeclaration
	: KWCLASS TYPEIDENTIFIER { PushCodeClass($2,@1,@2); } ClassInheritance Block {UpdateCodeClass(@5);}					
    | KWABSTRACT KWCLASS TYPEIDENTIFIER { PushCodeClass($2,@1,@2); } ClassInheritance Block {UpdateCodeClass(@6);}
    | KWINTERFACE TYPEIDENTIFIER { PushCodeInterface($2,@1,@2); } ClassInheritance Block
    | Function
	| VariableDeclarationStatement ';'
    | KWGLOBAL VARIABLE ';'											{ PushVariableDeclarationStatement(@1,@3,true); }
    | Include ComplexVariable ';'
	;
    
Include
	: KWINCLUDE
	| KWINCLUDEONCE
	| KWREQUIRE
	| KWREQUIREONCE
	;
	
String
	: SIMPLEQUOTE SIMPLEQUOTESTRING SIMPLEQUOTE
	| QUOTE QUOTESTRING QUOTE
	| SIMPLEQUOTESTRING
	| QUOTESTRING
	;

Block
    : OpenBlock CloseBlock      { Match(@1, @2); }
    | OpenBlock Declaration CloseBlock
                                { Match(@1, @3); }
    | OpenBlock Declaration error 
                                { CallHdlr("missing '}'", @3); }
    | OpenBlock error CloseBlock
                                { Match(@1, @3); }
    ;

OpenBlock
    : '{'                       { /*  */ }
    ;

CloseBlock
    : '}'                       { /*  */ }
    ;
  
ClassInheritance
	: INHERIT TypeParameter { UpdateBaseType($2.str); } IMPLEMENT TypeParameters { UpdateInterfaces($5.str); }
	| IMPLEMENT TypeParameters { UpdateInterfaces($2.str); }
	| INHERIT TypeParameter { UpdateBaseType($2.str); }
	|
	;
	
TypeParameters
	: TypeParameter
	| TypeParameters ',' TypeParameter
	;
	
TypeParameter
	: TYPEREFERENCEIDENTIFIER
	;
	
FunctionDeclaration
	: Visibility_opt KWFUNCTION IDENTIFIER { Sink.StartName(MkTSpan(@3),$3.str); } '(' { Sink.StartParameters(MkTSpan(@4)); } Parameters1 ')' { Sink.EndParameters(MkTSpan(@6));$$=$3; }
	;
	
Function
	: FunctionDeclaration Block { PushCodeFunction($1,false,@1,@2); }
	| FunctionDeclaration ';' { PushCodeFunction($1,true,@1,@2); }
	;
	
Parameters1
	: Parameters
	| 
	;
	
Parameters
	: Parameter
	| Parameters ',' Parameter
	;
	
Parameter
	: VARIABLE
	| IDENTIFIER VARIABLE
	| '&' VARIABLE
	| IDENTIFIER '&' VARIABLE
	; 
	
Visibility
	: KWPUBLIC
	| KWPROTECTED
	| KWPRIVATE
	;
	
Visibility_opt
	: Visibility
	|
	;
	
Statements
	: Statement
	| Statements Statement		
	;

Statement
	: IfStatement
	| VariableStatement ';'
	| ForStatement
	| ForEachStatement
	| SwitchStatement
	| WhileStatement
	| ReturnStatement ';'
	| ThrowStatement ';'
	| LEX_COMMENT
	;
	
SwitchStatement
	: KWSWITCH '(' ComplexVariable ')' '{' CaseBlocks '}'
	;
	
CaseBlocks
	: CaseBlocks CaseBlock
	| CaseBlock
	;
	
CaseBlock
	: KWCASE ComplexVariable ':' Declaration KWBREAK ';'
	| KWDEFAULT ':' Declaration KWBREAK ';'
	;

ThrowStatement
	:	KWTHROW ComplexVariable
	;
	
VariableStatement
	: ComplexVariable '=' Condition						{ PushAssignStatement(@1,@3); }
	| ComplexVariable SELFCONCAT ComplexVariable		{ PushAssignStatement(@1,@3); }
	| ComplexVariable
	;


ReturnStatement
	: KWRETURN ComplexVariable_opt					{ PushReturnStatement(@1,@1); }
	;

AssignationStatement
	: ComplexVariable '=' ComplexVariable			
	| ComplexVariable SELFCONCAT ComplexVariable	{ PushAssignStatement(@1,@3); }
	;
		
VariableDeclarationStatement
	: Visibility_opt VARIABLE Init_Opt				{ UpdateVisibility($1,@1,@3); }
	| Visibility_opt VARIABLE Init_Opt				{ UpdateVisibility($1,@1,@3); }
	| KWVAR VARIABLE Init_Opt						{ UpdateVisibility($1,@1,@3); }
	;
		
Init_Opt
	: '=' ComplexVariable
	|
	;
		
Arguments
	:	ComplexVariable
	|	Arguments ',' ComplexVariable
	;
	
Arguments_opt
	: Arguments
	|
	;
	
OneLineBlock
	: Block
	| Statement
	;
	
ForStatement
	: KWFOR '(' AssignationStatement ';' Condition ';' AssignationStatement ')' OneLineBlock  { PushForStatement(@1,@9); }
	;

ForEachStatement
	: KWFOREACH '(' ComplexVariable KWAS VARIABLE MAP VARIABLE ')' OneLineBlock
	| KWFOREACH '(' ComplexVariable KWAS VARIABLE ')' OneLineBlock
	;

WhileStatement
	: KWWHILE '(' Condition ')' OneLineBlock  { PushWhileStatement(@1,@5); }
	;
	
IfStatement
	: KWIF '(' Condition ')' { Match(@2,@4); } OneLineBlock ElseStatement	{ Match(@1,@7); PushIfStatement(@1,@7); }
	;
	
ElseStatement
	: KWELSE OneLineBlock { PushElseExpression(@1,@2); }
	| KWELSEIF '(' Condition ')' { Match(@2,@4); } OneLineBlock ElseStatement
	|
	;
	
Condition
	: Condition EQ Condition						{ PushConditionExpression(@1,@3,CodeConditionOperator.Equals); }
	| Condition TYPEEQ ComplexVariable					{ PushConditionExpression(@1,@3,CodeConditionOperator.Equals); }
	| ComplexVariable KWINSTANCEOF Identifier					{ PushConditionExpression(@1,@3,CodeConditionOperator.Equals); }
	| Condition NEQ Condition						{ PushConditionExpression(@1,@3,CodeConditionOperator.NotEquals); }
	| Condition GT Condition						{ PushConditionExpression(@1,@3,CodeConditionOperator.Greater); }
	| Condition GTE Condition						{ PushConditionExpression(@1,@3,CodeConditionOperator.GreaterOrEqual); }
	| Condition LT Condition						{ PushConditionExpression(@1,@3,CodeConditionOperator.Lesser); }
	| Condition LTE Condition						{ PushConditionExpression(@1,@3,CodeConditionOperator.LesserOrEqual); }
	| ComplexVariable											{ PushConditionExpression(@1,@1); }
	| ComplexVariable '=' ComplexVariable
	| '!' Condition												{ PushNotCondition(@1,@2); }
	| '(' Condition ')'
	| Condition AMPAMP Condition								{ PushConditionExpression(@1,@2,CodeConditionOperator.And); }
	| Condition BARBAR Condition								{ PushConditionExpression(@1,@2,CodeConditionOperator.Or); }
	;


CallStatement
	: MEMBERIDENTIFIER '(' Arguments_opt ')'					{ Match(@2,@4); PushCallStatement($1.str,false,@1,@4); WriteTrace($1.str,"CallStatement1"); }
	| BUILTIN_FUNCTION '(' Arguments_opt ')'					{ PushCallStatement($1.str,true,@1,@4); }
	| IDENTIFIER '(' Arguments_opt ')'							{ PushCallStatement($1.str,false,@1,@4); }
	| KWECHO ComplexVariable
	;
	
IndexerStatement
	: VARIABLE '[' ComplexVariable ']'							{ Match(@2,@4); PushIndexerStatement(@1,@4); }
	| MEMBERIDENTIFIER '[' ComplexVariable ']'					{ Match(@2,@4); PushIndexerStatement(@1,@4); }
	;
	
BasicVariable_
	: CallStatement
	| IndexerStatement
	| MEMBERIDENTIFIER
	;

Variable
	: VARIABLE												{ PushVariable($1,@1); }
	| '&' VARIABLE											{ PushVariable($1,@1); }
	| KWTHIS												{ PushThis(@1); }
	| '&' KWTHIS											{ PushThis(@1); }
	| KWNULL
	| String
	| IDENTIFIER											{ PushConstant($1,@1); }
	| KWNEW IDENTIFIER '(' Arguments_opt ')'				{ Match(@3,@5); }
	| KWNEW IDENTIFIER
	| KWTRUE												{ PushConstant($1,@1); }
	| KWFALSE												{ PushConstant($1,@1); }
	| NUMBER
	;
	
ComplexVariable_
	: KWPARENT CLASSACCESSOR CompositeVariable				{ PushClassAccessExpression($1,@1,@3); }
	| IDENTIFIER CLASSACCESSOR CompositeVariable			{ PushClassAccessExpression($1,@1,@3); }
	| FullCompositeVariable
	;
	
ComplexVariable
	: ComplexVariable '-' ComplexVariable
	| ComplexVariable '+' ComplexVariable
	| ComplexVariable '*' ComplexVariable
	| ComplexVariable '/' ComplexVariable
	| ComplexVariable '%' ComplexVariable
	| ComplexVariable '.' ComplexVariable
	| '(' ComplexVariable ')'								{ Match(@1,@3); }
	| ComplexVariable_
	;

ComplexVariable_opt
	: ComplexVariable
	|
	;
	
CompositeVariable
	: BasicVariable_ ACCESSOR CompositeVariable 			{ PushAccessExpression($3,@1,@3); }
	| BasicVariable_										{ WriteTrace(string.Concat("Variable reference : ",$1.str),"CompositeVariable1"); }
	;
	
CompositeVariable_
	: Variable ACCESSOR CompositeVariable					{ WriteTrace(string.Concat("Variable reference : ",$1.str),"CompositeVariable_0"); }
	| Variable												{ WriteTrace(string.Concat("Variable reference : ",$1.str),"CompositeVariable_1"); }
	;
	
FullCompositeVariable
	: CompositeVariable_
	| CompositeVariable										
	;


Identifier
	: VARIABLE												{ PushVariable($1,@1); }
	| IDENTIFIER											{ PushConstant($1,@1); }
	;

%%



